<?php

/**
 * This file contains QUI\LogXMPP\XMPPClient
 */
namespace QUI\LogXMPP;

use BirknerAlex\XMPPHP\XMPP;

/**
 * Class XMPPClient
 *
 * @package QUI\LogXMPP
 */
class XMPPClient extends XMPP
{

    /**
     * Connect to XMPP Host
     *
     * @param integer $timeout Timeout in seconds
     * @param boolean $persistent
     * @param boolean $sendinit Send XMPP starting sequence after connect
     *                            automatically
     *
     * @throws \Exception When the connection fails
     */
    public function connect($timeout = 30, $persistent = false, $sendinit = true)
    {
        $this->sent_disconnect = false;
        $starttime             = time();

        do {
            $this->disconnected    = false;
            $this->sent_disconnect = false;

            if ($persistent) {
                $conflag = STREAM_CLIENT_CONNECT | STREAM_CLIENT_PERSISTENT;
            } else {
                $conflag = STREAM_CLIENT_CONNECT;
            }

            $conntype = 'tcp';

            if ($this->use_ssl) {
                $conntype = 'ssl';
            }

            $this->log->log("Connecting to $conntype://{$this->host}:{$this->port}");

            try {
                // ssl options
                $options = array(
                    'ssl' => array(
                        'verify_peer'       => false,
                        'verify_peer_name'  => false,
                        'allow_self_signed' => true
                    )
                );
                $context = stream_context_create($options);

                $this->socket = @stream_socket_client(
                    "$conntype://{$this->host}:{$this->port}",
                    $errno,
                    $errstr,
                    $timeout,
                    $conflag,
                    $context
                );
            } catch (\Exception $Exception) {
                throw new \Exception($Exception->getMessage());
            }

            if (!$this->socket) {
                $this->log->log(
                    "Could not connect.",
                    \BirknerAlex\XMPPHP\Log::LEVEL_ERROR
                );

                $this->disconnected = true;
                # Take it easy for a few seconds
                sleep(min($timeout, 5));
            }

        } while (!$this->socket && (time() - $starttime) < $timeout);

        if ($this->socket) {
            stream_set_blocking($this->socket, 1);
            if ($sendinit) {
                $this->send($this->stream_start);
            }
        } else {
            throw new \Exception("Could not connect before timeout.");
        }
    }

    /**
     * @param string $xml
     */
    protected function tls_proceed_handler($xml)
    {
        $this->log->log("Starting TLS encryption");

        stream_socket_enable_crypto(
            $this->socket,
            true,
            STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT
        );

        //STREAM_CRYPTO_METHOD_SSLv23_CLIENT);
        //STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT

        $this->reset();
    }
}

